import React, { useCallback, useState } from 'react';
import { StyleSheet, TouchableOpacity, View, Text } from 'react-native';
import ActionSheet, { SheetManager } from 'react-native-actions-sheet';
import { getFontSize } from 'src/utils';
import { Colors } from 'src/theme';
import { WarningProps } from '../types';
import { useSafeAreaInsets } from 'react-native-safe-area-context';
import { usePostReportConversationMutation } from '@api/chat';
import * as ImagePicker from 'expo-image-picker';
import * as DocumentPicker from 'react-native-document-picker';

import MegaphoneIcon from 'assets/icons/messages/megaphone.svg';
import { MaterialCommunityIcons } from '@expo/vector-icons';

const AttachmentsModal = () => {
  const insets = useSafeAreaInsets();
  const [chatData, setChatData] = useState<any>(null);
  const [shouldOpenWarningModal, setShouldOpenWarningModal] = useState<WarningProps | null>(null);
  const { mutateAsync: reportUser } = usePostReportConversationMutation();

  const handleSheetOpen = (payload: any) => {
    setChatData(payload);
  };

  const handleReport = async () => {
    if (!chatData) return;

    setShouldOpenWarningModal({
      title: `Report ${chatData.name}`,
      buttonTitle: 'Report',
      message: `Are you sure you want to report ${chatData.name}?\nIf you proceed, the chat history with ${chatData.name} will become visible to NomadMania admins for investigation.`,
      action: async () => {
        await reportUser({
          token: chatData.token,
          reported_user_id: chatData.uid
        });
      }
    });

    setTimeout(() => {
      SheetManager.hide('chat-attachments');
      setShouldOpenWarningModal(null);
    }, 300);
  };

  const handleOpenGallery = useCallback(async () => {
    if (!chatData) return;
    try {
      const perm = await ImagePicker.requestMediaLibraryPermissionsAsync();
      if (!perm.granted) {
        console.warn('Permission for gallery not granted');
        return;
      }

      const result = await ImagePicker.launchImageLibraryAsync({
        mediaTypes: ImagePicker.MediaTypeOptions.All,
        allowsMultipleSelection: true,
        quality: 1,
        selectionLimit: 4
      });

      if (!result.canceled && result.assets) {
        const files = result.assets.map((asset) => ({
          uri: asset.uri,
          type: asset.type === 'video' ? 'video' : 'image'
        }));
        chatData.onSendMedia(files);
      }
      SheetManager.hide('chat-attachments');
    } catch (err) {
      console.warn('Gallery error: ', err);
    }
  }, [chatData?.onSendMedia, chatData?.closeOptions]);

  const handleOpenCamera = useCallback(async () => {
    if (!chatData) return;
    try {
      const perm = await ImagePicker.requestCameraPermissionsAsync();
      if (!perm.granted) {
        console.warn('Permission for camera not granted');
        return;
      }

      const result = await ImagePicker.launchCameraAsync({
        mediaTypes: ImagePicker.MediaTypeOptions.Images,
        quality: 1
      });

      if (!result.canceled && result.assets) {
        const files = result.assets.map((asset) => ({
          uri: asset.uri,
          type: asset.type === 'video' ? 'video' : 'image'
        }));
        chatData.onSendMedia(files);
      }
      SheetManager.hide('chat-attachments');
    } catch (err) {
      console.warn('Camera error: ', err);
    }
  }, [chatData?.onSendMedia, chatData?.closeOptions]);

  const handleShareLocation = useCallback(async () => {
    if (!chatData) return;
    try {
      // TODO:
      // const { status } = await Location.requestForegroundPermissionsAsync();
      // if (status !== 'granted') {}

      // const loc = await Location.getCurrentPositionAsync({});
      // const coords = { latitude: loc.coords.latitude, longitude: loc.coords.longitude };

      const coords = { latitude: 50.4501, longitude: 30.5234 };
      chatData.onSendLocation(coords);
      SheetManager.hide('chat-attachments');
    } catch (err) {
      console.warn('Location error: ', err);
    }
  }, [chatData?.onSendLocation, chatData?.closeOptions]);

  const handleShareLiveLocation = useCallback(() => {
    if (!chatData) return;
    chatData.onShareLiveLocation();
    SheetManager.hide('chat-attachments');
  }, [chatData?.onShareLiveLocation, chatData?.closeOptions]);

  const handleSendFile = useCallback(async () => {
    if (!chatData) return;

    try {
      const res = await DocumentPicker.pick({
        type: [DocumentPicker.types.allFiles],
        allowMultiSelection: false
      });

      const file = {
        uri: res[0].uri,
        name: res[0].name,
        type: res[0].type
      };

      if (chatData.onSendFile) {
        chatData.onSendFile([file]);
      }
    } catch (err) {
      if (DocumentPicker.isCancel(err)) {
        console.log('User canceled document picker');
      } else {
        console.warn('DocumentPicker error:', err);
      }
    }

    SheetManager.hide('chat-attachments');
  }, [chatData?.onSendFile, chatData?.closeOptions]);

  return (
    <ActionSheet
      id="chat-attachments"
      gestureEnabled={true}
      containerStyle={{
        borderTopLeftRadius: 15,
        borderTopRightRadius: 15
      }}
      defaultOverlayOpacity={0.3}
      onBeforeShow={(sheetRef) => {
        const payload = sheetRef || null;
        handleSheetOpen(payload);
      }}
      onClose={() => {
        if (shouldOpenWarningModal) {
          chatData?.setModalInfo({
            visible: true,
            type: 'delete',
            title: shouldOpenWarningModal.title,
            buttonTitle: shouldOpenWarningModal.buttonTitle,
            message: shouldOpenWarningModal.message,
            action: shouldOpenWarningModal.action
          });
        }
      }}
    >
      {/* <View
        style={{
          backgroundColor: 'white',
          paddingHorizontal: 16,
          gap: 16,
          paddingTop: 8,
          paddingBottom: 8 + insets.bottom
        }}
      >
        <TouchableOpacity style={[styles.option]} onPress={handleReport}>
          <Text style={[styles.optionText]}>Gallery</Text>
          <MegaphoneIcon fill={Colors.DARK_BLUE} />
        </TouchableOpacity>

        <TouchableOpacity style={[styles.option]} onPress={handleReport}>
          <Text style={[styles.optionText]}>Camera</Text>
          <MegaphoneIcon fill={Colors.DARK_BLUE} />
        </TouchableOpacity>

        <TouchableOpacity style={[styles.option]} onPress={handleReport}>
          <Text style={[styles.optionText]}>Location</Text>
          <MegaphoneIcon fill={Colors.DARK_BLUE} />
        </TouchableOpacity>

        <TouchableOpacity style={[styles.option, styles.dangerOption]} onPress={handleReport}>
          <Text style={[styles.optionText, styles.dangerText]}>Report {chatData?.name}</Text>
          <MegaphoneIcon fill={Colors.RED} />
        </TouchableOpacity>
      </View> */}
      <View style={[styles.container, { paddingBottom: 8 + insets.bottom }]}>
        <View style={styles.optionRow}>
          <TouchableOpacity style={styles.optionItem} onPress={handleOpenGallery}>
            <MaterialCommunityIcons name="image" size={36} color={Colors.ORANGE} />
            <Text style={styles.optionLabel}>Gallery</Text>
          </TouchableOpacity>

          <TouchableOpacity style={styles.optionItem} onPress={handleOpenCamera}>
            <MaterialCommunityIcons name="camera" size={36} color={Colors.ORANGE} />
            <Text style={styles.optionLabel}>Camera</Text>
          </TouchableOpacity>

          <TouchableOpacity style={styles.optionItem} onPress={handleShareLocation}>
            <MaterialCommunityIcons name="map-marker" size={36} color={Colors.ORANGE} />
            <Text style={styles.optionLabel}>Location</Text>
          </TouchableOpacity>

          <TouchableOpacity style={styles.optionItem} onPress={handleShareLiveLocation}>
            <MaterialCommunityIcons name="navigation" size={36} color={Colors.ORANGE} />
            <Text style={styles.optionLabel}>Live</Text>
          </TouchableOpacity>

          <TouchableOpacity style={styles.optionItem} onPress={handleSendFile}>
            <MaterialCommunityIcons name="file" size={36} color={Colors.ORANGE} />
            <Text style={styles.optionLabel}>File</Text>
          </TouchableOpacity>

          <TouchableOpacity style={styles.optionItem} onPress={handleReport}>
            <MegaphoneIcon fill={Colors.RED} width={36} height={36} />
            <Text style={styles.optionLabel}>Report</Text>
          </TouchableOpacity>
          <View style={styles.optionItem}></View>
        </View>
      </View>
    </ActionSheet>
  );
};

const styles = StyleSheet.create({
  option: {
    flexDirection: 'row',
    alignItems: 'center',
    justifyContent: 'space-between'
  },
  optionText: {
    fontSize: getFontSize(12),
    fontWeight: '600',
    color: Colors.DARK_BLUE
  },
  dangerOption: {
    paddingVertical: 10,
    borderBottomWidth: 1,
    borderBlockColor: Colors.WHITE
  },
  dangerText: {
    color: Colors.RED
  },
  container: {
    backgroundColor: Colors.WHITE
  },
  optionRow: {
    flexDirection: 'row',
    justifyContent: 'space-between',
    paddingHorizontal: '5%',
    marginVertical: 20,
    flexWrap: 'wrap'
  },
  optionItem: {
    width: '30%',
    paddingVertical: 8,
    marginBottom: 12,
    alignItems: 'center'
  },
  optionLabel: {
    marginTop: 6,
    fontSize: 12,
    color: Colors.DARK_BLUE,
    fontWeight: '700'
  }
});

export default AttachmentsModal;
